@@ -49,8 +49,6 @@
 #include <linux/rtnetlink.h>
 #ifdef HAVE_LINUX_WIRELESS_H
 #include <linux/wireless.h>
-#else
-#include <net/if.h>
 #endif
 #include <sys/ioctl.h>
 #include <errno.h>
@@ -62,10 +60,6 @@
 #include "gupnp-linux-context-manager.h"
 #include "gupnp-context.h"
 
-G_DEFINE_TYPE (GUPnPLinuxContextManager,
-               gupnp_linux_context_manager,
-               GUPNP_TYPE_CONTEXT_MANAGER);
-
 struct _GUPnPLinuxContextManagerPrivate {
         /* Socket used for IOCTL calls */
         int fd;
@@ -86,11 +80,18 @@ struct _GUPnPLinuxContextManagerPrivate {
          * structure */
         GHashTable *interfaces;
 
-        /* Receive buffer for netlink messages. */
-        char recvbuf[8196];
-
         gboolean dump_netlink_packets;
 };
+typedef struct _GUPnPLinuxContextManagerPrivate GUPnPLinuxContextManagerPrivate;
+
+
+struct _GUPnPLinuxContextManager {
+        GUPnPContextManager parent;
+};
+
+G_DEFINE_TYPE_WITH_PRIVATE (GUPnPLinuxContextManager,
+                            gupnp_linux_context_manager,
+                            GUPNP_TYPE_CONTEXT_MANAGER);
 
 typedef enum {
         /* Interface is up */
@@ -104,7 +105,7 @@ typedef enum {
 } NetworkInterfaceFlags;
 
 static void
-dump_rta_attr (struct rtattr *rt_attr)
+dump_rta_attr (sa_family_t family, struct rtattr *rt_attr)
 {
         const char *data = NULL;
         const char *label = NULL;
@@ -114,10 +115,10 @@ dump_rta_attr (struct rtattr *rt_attr)
                         rt_attr->rta_type == IFA_LOCAL ||
                         rt_attr->rta_type == IFA_BROADCAST ||
                         rt_attr->rta_type == IFA_ANYCAST) {
-                data = inet_ntop (AF_INET,
-                                RTA_DATA (rt_attr),
-                                buf,
-                                sizeof (buf));
+                data = inet_ntop (family,
+                                  RTA_DATA (rt_attr),
+                                  buf,
+                                  sizeof (buf));
         } else if (rt_attr->rta_type == IFA_LABEL) {
                 data = (const char *) RTA_DATA (rt_attr);
         } else {
@@ -225,6 +226,16 @@ network_device_new (GUPnPLinuxContextManager *manager,
         return device;
 }
 
+static int
+get_ioctl_fd (GUPnPLinuxContextManager *self)
+{
+    GUPnPLinuxContextManagerPrivate *priv;
+
+    priv = gupnp_linux_context_manager_get_instance_private (self);
+
+    return priv->fd;
+}
+
 /* Try to update the ESSID of a network interface. */
 static void
 network_device_update_essid (NetworkInterface *device)
@@ -241,7 +252,7 @@ network_device_update_essid (NetworkInterface *device)
         strncpy (iwr.ifr_name, device->name, IFNAMSIZ - 1);
         iwr.u.essid.pointer = (caddr_t) essid;
         iwr.u.essid.length = IW_ESSID_MAX_SIZE;
-        ret = ioctl (device->manager->priv->fd, SIOCGIWESSID, &iwr);
+        ret = ioctl (get_ioctl_fd (device->manager), SIOCGIWESSID, &iwr);
 
         if ((ret == 0 && essid[0] != '\0') &&
             (!device->essid || strcmp (device->essid, essid)))
@@ -407,6 +418,7 @@ on_netlink_message_available (G_GNUC_UNUSED GSocket     *socket,
 static void
 extract_info (struct nlmsghdr *header,
               gboolean         dump,
+              sa_family_t      family,
               guint8           prefixlen,
               char           **address,
               char           **label,
@@ -420,30 +432,53 @@ extract_info (struct nlmsghdr *header,
         rt_attr_len = IFA_PAYLOAD (header);
         while (RT_ATTR_OK (rt_attr, rt_attr_len)) {
                 if (dump) {
-                        dump_rta_attr (rt_attr);
+                        dump_rta_attr (family, rt_attr);
                 }
 
                 if (rt_attr->rta_type == IFA_LABEL) {
                         *label = g_strdup ((char *) RTA_DATA (rt_attr));
-                } else if (rt_attr->rta_type == IFA_LOCAL) {
+                } else if (rt_attr->rta_type == IFA_ADDRESS) {
                         if (address != NULL) {
-                                inet_ntop (AF_INET,
+                                const char *result = NULL;
+                                result = inet_ntop (family,
                                            RTA_DATA (rt_attr),
                                            buf,
                                            sizeof (buf));
-                                *address = g_strdup (buf);
+                                if (result == NULL) {
+                                    g_critical ("Failed ntop: %m");
+                                } else {
+                                    *address = g_strdup (result);
+                                }
                         }
 
                         if (mask != NULL) {
-                                struct in_addr addr, *data;
-                                guint32 bitmask;
+                                struct in6_addr addr = { 0 }, *data;
+                                int i = 0, bits = prefixlen;
+                                guint8 *outbuf = (guint8 *)&addr.s6_addr;
+                                guint8 *inbuf;
 
-                                bitmask = htonl(G_MAXUINT32 << (32 - prefixlen));
                                 data = RTA_DATA (rt_attr);
+                                inbuf = (guint8 *)&data->s6_addr;
 
-                                addr.s_addr = data->s_addr & bitmask;
+                                for (i = 0; i < (family == AF_INET ? 4 : 16); i++) {
+                                        if (bits > 8) {
+                                                bits -= 8;
+                                                outbuf[i] = inbuf[i] & 0xff;
+                                        } else {
+                                                static const guint8 bits_a[] =
+                                                { 0x00, 0x08, 0x0C, 0x0E, 0x0F };
 
-                                inet_ntop (AF_INET,
+                                                if (bits >= 4) {
+                                                        outbuf[i] = inbuf[i] & 0xf0;
+                                                        bits -= 4;
+                                                }
+                                                outbuf[i] = outbuf[i] |
+                                                            (inbuf[i] & bits_a[bits]);
+                                                break;
+                                        }
+                                }
+
+                                inet_ntop (family,
                                            &addr,
                                            buf,
                                            sizeof (buf));
@@ -492,8 +527,10 @@ create_context (GUPnPLinuxContextManager *self,
                 struct ifaddrmsg         *ifa)
 {
         NetworkInterface *device;
+        GUPnPLinuxContextManagerPrivate *priv;
 
-        device = g_hash_table_lookup (self->priv->interfaces,
+        priv = gupnp_linux_context_manager_get_instance_private (self);
+        device = g_hash_table_lookup (priv->interfaces,
                                       GINT_TO_POINTER (ifa->ifa_index));
 
         if (!device) {
@@ -522,8 +559,10 @@ remove_context (GUPnPLinuxContextManager *self,
 {
         NetworkInterface *device;
         GUPnPContext *context;
+        GUPnPLinuxContextManagerPrivate *priv;
 
-        device = g_hash_table_lookup (self->priv->interfaces,
+        priv = gupnp_linux_context_manager_get_instance_private (self);
+        device = g_hash_table_lookup (priv->interfaces,
                                       GINT_TO_POINTER (ifa->ifa_index));
 
         if (!device) {
@@ -561,24 +600,27 @@ remove_context (GUPnPLinuxContextManager *self,
 static gboolean
 on_bootstrap (GUPnPLinuxContextManager *self)
 {
-        if (self->priv->nl_seq == 0) {
+        GUPnPLinuxContextManagerPrivate *priv;
+
+        priv = gupnp_linux_context_manager_get_instance_private (self);
+        if (priv->nl_seq == 0) {
                 query_all_network_interfaces (self);
 
                 return TRUE;
-        } else if (self->priv->nl_seq == 1) {
+        } else if (priv->nl_seq == 1) {
                 query_all_addresses (self);
 
                 return TRUE;
         } else {
-                self->priv->netlink_socket_source = g_socket_create_source
-                                                (self->priv->netlink_socket,
+                priv->netlink_socket_source = g_socket_create_source
+                                                (priv->netlink_socket,
                                                  G_IO_IN | G_IO_PRI,
                                                  NULL);
 
-                g_source_attach (self->priv->netlink_socket_source,
+                g_source_attach (priv->netlink_socket_source,
                                  g_main_context_get_thread_default ());
 
-                g_source_set_callback (self->priv->netlink_socket_source,
+                g_source_set_callback (priv->netlink_socket_source,
                                        (GSourceFunc)
                                        on_netlink_message_available,
                                        self,
@@ -603,6 +645,9 @@ send_netlink_request (GUPnPLinuxContextManager *self,
         struct msghdr msg;
         struct iovec io;
         int fd;
+        GUPnPLinuxContextManagerPrivate *priv;
+
+        priv = gupnp_linux_context_manager_get_instance_private (self);
 
         memset (&req, 0, sizeof (req));
         memset (&dest, 0, sizeof (dest));
@@ -610,10 +655,10 @@ send_netlink_request (GUPnPLinuxContextManager *self,
 
         dest.nl_family = AF_NETLINK;
         req.hdr.nlmsg_len = NLMSG_LENGTH (sizeof (struct rtgenmsg));
-        req.hdr.nlmsg_seq = self->priv->nl_seq++;
+        req.hdr.nlmsg_seq = priv->nl_seq++;
         req.hdr.nlmsg_type = netlink_message;
         req.hdr.nlmsg_flags = NLM_F_REQUEST | flags;
-        req.gen.rtgen_family = AF_INET;
+        req.gen.rtgen_family = gupnp_context_manager_get_socket_family (GUPNP_CONTEXT_MANAGER (self));
 
         io.iov_base = &req;
         io.iov_len = req.hdr.nlmsg_len;
@@ -623,7 +668,7 @@ send_netlink_request (GUPnPLinuxContextManager *self,
         msg.msg_name = &dest;
         msg.msg_namelen = sizeof (dest);
 
-        fd = g_socket_get_fd (self->priv->netlink_socket);
+        fd = g_socket_get_fd (priv->netlink_socket);
         if (sendmsg (fd, (struct msghdr *) &msg, 0) < 0)
                 g_warning ("Could not send netlink message: %s",
                            strerror (errno));
@@ -654,7 +699,7 @@ query_all_addresses (GUPnPLinuxContextManager *self)
         g_debug ("Bootstrap: Querying all addresses");
         send_netlink_request (self,
                               RTM_GETADDR,
-                              NLM_F_ROOT | NLM_F_MATCH | NLM_F_ACK);
+                              NLM_F_DUMP);
 }
 
 /* Ignore non-multicast device, except loop-back and P-t-P devices */
@@ -670,9 +715,12 @@ handle_device_status_change (GUPnPLinuxContextManager *self,
 {
         gpointer key;
         NetworkInterface *device;
+        GUPnPLinuxContextManagerPrivate *priv;
+
+        priv = gupnp_linux_context_manager_get_instance_private (self);
 
         key = GINT_TO_POINTER (ifi->ifi_index);
-        device = g_hash_table_lookup (self->priv->interfaces,
+        device = g_hash_table_lookup (priv->interfaces,
                                       key);
 
         if (device != NULL) {
@@ -691,7 +739,7 @@ handle_device_status_change (GUPnPLinuxContextManager *self,
                 if (ifi->ifi_flags & IFF_UP)
                         device->flags |= NETWORK_INTERFACE_UP;
 
-                g_hash_table_insert (self->priv->interfaces,
+                g_hash_table_insert (priv->interfaces,
                                      key,
                                      device);
         }
@@ -701,7 +749,11 @@ static void
 remove_device (GUPnPLinuxContextManager *self,
                struct ifinfomsg         *ifi)
 {
-        g_hash_table_remove (self->priv->interfaces,
+        GUPnPLinuxContextManagerPrivate *priv;
+
+        priv = gupnp_linux_context_manager_get_instance_private (self);
+
+        g_hash_table_remove (priv->interfaces,
                              GINT_TO_POINTER (ifi->ifi_index));
 }
 
@@ -713,17 +765,20 @@ remove_device (GUPnPLinuxContextManager *self,
 static void
 receive_netlink_message (GUPnPLinuxContextManager *self, GError **error)
 {
+        static char buf[8196];
+
         gssize len;
         GError *inner_error = NULL;
-        struct nlmsghdr *header;
+        struct nlmsghdr *header = (struct nlmsghdr *) buf;
         struct ifinfomsg *ifi;
         struct ifaddrmsg *ifa;
+        GUPnPLinuxContextManagerPrivate *priv;
 
-        header = (struct nlmsghdr *) self->priv->recvbuf;
-        len = g_socket_receive (self->priv->netlink_socket,
+        priv = gupnp_linux_context_manager_get_instance_private (self);
 
-                                self->priv->recvbuf,
-                                sizeof (self->priv->recvbuf),
+        len = g_socket_receive (priv->netlink_socket,
+                                buf,
+                                sizeof (buf),
                                 NULL,
                                 &inner_error);
         if (len == -1) {
@@ -735,13 +790,12 @@ receive_netlink_message (GUPnPLinuxContextManager *self, GError **error)
                 return;
         }
 
-        if (self->priv->dump_netlink_packets) {
+        if (priv->dump_netlink_packets) {
                 GString *hexdump = NULL;
 
                 /* We should have at most len / 16 + 1 lines with 74 characters each */
                 hexdump = g_string_new_len (NULL, ((len / 16) + 1) * 73);
-                gupnp_linux_context_manager_hexdump ((guint8 *) self->priv->recvbuf,
-                                                     len, hexdump);
+                gupnp_linux_context_manager_hexdump ((guint8 *) buf, len, hexdump);
 
                 g_debug ("Netlink packet dump:\n%s", hexdump->str);
                 g_string_free (hexdump, TRUE);
@@ -765,13 +819,22 @@ receive_netlink_message (GUPnPLinuxContextManager *self, GError **error)
 
                                 g_debug ("Received RTM_NEWADDR");
                                 ifa = NLMSG_DATA (header);
+                                if (ifa->ifa_family == AF_INET6 &&
+                                        (ifa->ifa_scope != RT_SCOPE_SITE &&
+                                         ifa->ifa_scope != RT_SCOPE_LINK &&
+                                         ifa->ifa_scope != RT_SCOPE_HOST))
+                                        break;
+
                                 extract_info (header,
-                                              self->priv->dump_netlink_packets,
+                                              priv->dump_netlink_packets,
+                                              ifa->ifa_family,
                                               ifa->ifa_prefixlen,
                                               &address,
                                               &label,
                                               &mask);
-                                create_context (self, address, label, mask, ifa);
+
+                                if (address != NULL)
+                                        create_context (self, address, label, mask, ifa);
                                 g_free (label);
                                 g_free (address);
                                 g_free (mask);
@@ -784,13 +847,22 @@ receive_netlink_message (GUPnPLinuxContextManager *self, GError **error)
 
                                 g_debug ("Received RTM_DELADDR");
                                 ifa = NLMSG_DATA (header);
+
+                                if (ifa->ifa_family == AF_INET6 &&
+                                        (ifa->ifa_scope != RT_SCOPE_SITE &&
+                                         ifa->ifa_scope != RT_SCOPE_LINK &&
+                                         ifa->ifa_scope != RT_SCOPE_HOST))
+                                        break;
+
                                 extract_info (header,
-                                              self->priv->dump_netlink_packets,
+                                              priv->dump_netlink_packets,
+                                              ifa->ifa_family,
                                               ifa->ifa_prefixlen,
                                               &address,
                                               &label,
                                               NULL);
-                                remove_context (self, address, label, ifa);
+                                if (address != NULL)
+                                        remove_context (self, address, label, ifa);
                                 g_free (label);
                                 g_free (address);
                             }
@@ -833,10 +905,14 @@ receive_netlink_message (GUPnPLinuxContextManager *self, GError **error)
 static gboolean
 create_ioctl_socket (GUPnPLinuxContextManager *self, GError **error)
 {
-        self->priv->fd = socket (AF_INET, SOCK_DGRAM, 0);
+        GUPnPLinuxContextManagerPrivate *priv;
 
-        if (self->priv->fd < 0) {
-                self->priv->fd = 0;
+        priv = gupnp_linux_context_manager_get_instance_private (self);
+
+        priv->fd = socket (AF_INET, SOCK_DGRAM, 0);
+
+        if (priv->fd < 0) {
+                priv->fd = 0;
 
                 g_set_error_literal (error,
                                      G_IO_ERROR,
@@ -857,6 +933,9 @@ create_netlink_socket (GUPnPLinuxContextManager *self, GError **error)
         int fd, status;
         GSocket *sock;
         GError *inner_error;
+        GUPnPLinuxContextManagerPrivate *priv;
+
+        priv = gupnp_linux_context_manager_get_instance_private (self);
 
         inner_error = NULL;
 
@@ -872,7 +951,14 @@ create_netlink_socket (GUPnPLinuxContextManager *self, GError **error)
         memset (&sa, 0, sizeof (sa));
         sa.nl_family = AF_NETLINK;
         /* Listen for interface changes and IP address changes */
-        sa.nl_groups = RTMGRP_LINK | RTMGRP_IPV4_IFADDR;
+        sa.nl_groups = RTMGRP_LINK;
+        if (gupnp_context_manager_get_socket_family (GUPNP_CONTEXT_MANAGER (self)) == G_SOCKET_FAMILY_INVALID) {
+                sa.nl_groups = RTMGRP_IPV6_IFADDR | RTMGRP_IPV4_IFADDR;
+        } else if (gupnp_context_manager_get_socket_family (GUPNP_CONTEXT_MANAGER (self)) == G_SOCKET_FAMILY_IPV4) {
+                sa.nl_groups = RTMGRP_IPV4_IFADDR;
+        } else if (gupnp_context_manager_get_socket_family (GUPNP_CONTEXT_MANAGER (self)) == G_SOCKET_FAMILY_IPV6) {
+                sa.nl_groups = RTMGRP_IPV6_IFADDR;
+        }
 
         status = bind (fd, (struct sockaddr *) &sa, sizeof (sa));
         if (status == -1) {
@@ -898,7 +984,7 @@ create_netlink_socket (GUPnPLinuxContextManager *self, GError **error)
 
         g_socket_set_blocking (sock, FALSE);
 
-        self->priv->netlink_socket = sock;
+        priv->netlink_socket = sock;
 
         return TRUE;
 }
@@ -925,14 +1011,13 @@ gupnp_linux_context_manager_is_available (void)
 static void
 gupnp_linux_context_manager_init (GUPnPLinuxContextManager *self)
 {
-        self->priv =
-                G_TYPE_INSTANCE_GET_PRIVATE (self,
-                                             GUPNP_TYPE_LINUX_CONTEXT_MANAGER,
-                                             GUPnPLinuxContextManagerPrivate);
+        GUPnPLinuxContextManagerPrivate *priv;
 
-        self->priv->nl_seq = 0;
+        priv = gupnp_linux_context_manager_get_instance_private (self);
 
-        self->priv->interfaces =
+        priv->nl_seq = 0;
+
+        priv->interfaces =
                 g_hash_table_new_full (g_direct_hash,
                                        g_direct_equal,
                                        NULL,
@@ -947,11 +1032,13 @@ gupnp_linux_context_manager_constructed (GObject *object)
         GUPnPLinuxContextManager *self;
         GError *error = NULL;
         const char *env_var = NULL;
+        GUPnPLinuxContextManagerPrivate *priv;
 
         self = GUPNP_LINUX_CONTEXT_MANAGER (object);
+        priv = gupnp_linux_context_manager_get_instance_private (self);
 
         env_var = g_getenv ("GUPNP_DEBUG_NETLINK");
-        self->priv->dump_netlink_packets = (env_var != NULL) &&
+        priv->dump_netlink_packets = (env_var != NULL) &&
                 strstr (env_var, "1") != NULL;
 
         if (!create_ioctl_socket (self, &error))
@@ -960,17 +1047,17 @@ gupnp_linux_context_manager_constructed (GObject *object)
         if (!create_netlink_socket (self, &error))
                 goto cleanup;
 
-        self->priv->bootstrap_source = g_idle_source_new ();
-        g_source_attach (self->priv->bootstrap_source,
+        priv->bootstrap_source = g_idle_source_new ();
+        g_source_attach (priv->bootstrap_source,
                          g_main_context_get_thread_default ());
-        g_source_set_callback (self->priv->bootstrap_source,
+        g_source_set_callback (priv->bootstrap_source,
                                (GSourceFunc) on_bootstrap,
                                self,
                                NULL);
 cleanup:
         if (error) {
-                if (self->priv->fd > 0)
-                        close (self->priv->fd);
+                if (priv->fd > 0)
+                        close (priv->fd);
 
                 g_warning ("Failed to setup Linux context manager: %s",
                            error->message);
@@ -989,34 +1076,36 @@ gupnp_linux_context_manager_dispose (GObject *object)
 {
         GUPnPLinuxContextManager *self;
         GObjectClass *parent_class;
+        GUPnPLinuxContextManagerPrivate *priv;
 
         self = GUPNP_LINUX_CONTEXT_MANAGER (object);
+        priv = gupnp_linux_context_manager_get_instance_private (self);
 
-        if (self->priv->bootstrap_source != NULL) {
-                g_source_destroy (self->priv->bootstrap_source);
-                g_source_unref (self->priv->bootstrap_source);
-                self->priv->bootstrap_source = NULL;
+        if (priv->bootstrap_source != NULL) {
+                g_source_destroy (priv->bootstrap_source);
+                g_source_unref (priv->bootstrap_source);
+                priv->bootstrap_source = NULL;
         }
 
-        if (self->priv->netlink_socket_source != NULL) {
-               g_source_destroy (self->priv->netlink_socket_source);
-               g_source_unref (self->priv->netlink_socket_source);
-               self->priv->netlink_socket_source = NULL;
+        if (priv->netlink_socket_source != NULL) {
+               g_source_destroy (priv->netlink_socket_source);
+               g_source_unref (priv->netlink_socket_source);
+               priv->netlink_socket_source = NULL;
         }
 
-        if (self->priv->netlink_socket != NULL) {
-                g_object_unref (self->priv->netlink_socket);
-                self->priv->netlink_socket = NULL;
+        if (priv->netlink_socket != NULL) {
+                g_object_unref (priv->netlink_socket);
+                priv->netlink_socket = NULL;
         }
 
-        if (self->priv->fd != 0) {
-                close (self->priv->fd);
-                self->priv->fd = 0;
+        if (priv->fd != 0) {
+                close (priv->fd);
+                priv->fd = 0;
         }
 
-        if (self->priv->interfaces) {
-                g_hash_table_destroy (self->priv->interfaces);
-                self->priv->interfaces = NULL;
+        if (priv->interfaces) {
+                g_hash_table_destroy (priv->interfaces);
+                priv->interfaces = NULL;
         }
 
         /* Chain-up */
@@ -1033,7 +1122,4 @@ gupnp_linux_context_manager_class_init (GUPnPLinuxContextManagerClass *klass)
 
         object_class->constructed = gupnp_linux_context_manager_constructed;
         object_class->dispose     = gupnp_linux_context_manager_dispose;
-
-        g_type_class_add_private (klass,
-                                  sizeof (GUPnPLinuxContextManagerPrivate));
 }
