@@ -19,56 +19,25 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef __GUPNP_SERVICE_PROXY_H__
-#define __GUPNP_SERVICE_PROXY_H__
+#ifndef GUPNP_SERVICE_PROXY_H
+#define GUPNP_SERVICE_PROXY_H
 
 #include "gupnp-error.h"
 #include "gupnp-service-info.h"
 
 G_BEGIN_DECLS
 
-GType
-gupnp_service_proxy_get_type (void) G_GNUC_CONST;
-
 GType
 gupnp_service_proxy_action_get_type (void) G_GNUC_CONST;
 
 #define GUPNP_TYPE_SERVICE_PROXY \
                 (gupnp_service_proxy_get_type ())
-#define GUPNP_SERVICE_PROXY(obj) \
-                (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
-                 GUPNP_TYPE_SERVICE_PROXY, \
-                 GUPnPServiceProxy))
-#define GUPNP_SERVICE_PROXY_CLASS(obj) \
-                (G_TYPE_CHECK_CLASS_CAST ((obj), \
-                 GUPNP_TYPE_SERVICE_PROXY, \
-                 GUPnPServiceProxyClass))
-#define GUPNP_IS_SERVICE_PROXY(obj) \
-                (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
-                 GUPNP_TYPE_SERVICE_PROXY))
-#define GUPNP_IS_SERVICE_PROXY_CLASS(obj) \
-                (G_TYPE_CHECK_CLASS_TYPE ((obj), \
-                 GUPNP_TYPE_SERVICE_PROXY))
-#define GUPNP_SERVICE_PROXY_GET_CLASS(obj) \
-                (G_TYPE_INSTANCE_GET_CLASS ((obj), \
-                 GUPNP_TYPE_SERVICE_PROXY, \
-                 GUPnPServiceProxyClass))
 
-typedef struct _GUPnPServiceProxyPrivate GUPnPServiceProxyPrivate;
-typedef struct _GUPnPServiceProxy GUPnPServiceProxy;
-typedef struct _GUPnPServiceProxyClass GUPnPServiceProxyClass;
-
-/**
- * GUPnPServiceProxy:
- *
- * This struct contains private data only, and should be accessed using the
- * functions below.
- */
-struct _GUPnPServiceProxy {
-        GUPnPServiceInfo parent;
-
-        GUPnPServiceProxyPrivate *priv;
-};
+G_DECLARE_DERIVABLE_TYPE (GUPnPServiceProxy,
+                          gupnp_service_proxy,
+                          GUPNP,
+                          SERVICE_PROXY,
+                          GUPnPServiceInfo)
 
 struct _GUPnPServiceProxyClass {
         GUPnPServiceInfoClass parent_class;
@@ -133,35 +102,15 @@ gupnp_service_proxy_send_action_valist
                                     GError                        **error,
                                     va_list                         var_args);
 
-#ifndef GUPNP_DISABLE_DEPRECATED
-gboolean
-gupnp_service_proxy_send_action_hash
-                                   (GUPnPServiceProxy              *proxy,
-                                    const char                     *action,
-                                    GError                        **error,
-                                    GHashTable                     *in_hash,
-                                    GHashTable                     *out_hash) G_GNUC_DEPRECATED;
-#endif
-
 gboolean
 gupnp_service_proxy_send_action_list (GUPnPServiceProxy *proxy,
                                       const char        *action,
-                                      GError           **error,
                                       GList             *in_names,
                                       GList             *in_values,
                                       GList             *out_names,
                                       GList             *out_types,
-                                      GList            **out_values);
-
-gboolean
-gupnp_service_proxy_send_action_list_gi (GUPnPServiceProxy *proxy,
-                                         const char        *action,
-                                         GList             *in_names,
-                                         GList             *in_values,
-                                         GList             *out_names,
-                                         GList             *out_types,
-                                         GList            **out_values,
-                                         GError           **error);
+                                      GList            **out_values,
+                                      GError           **error);
 GUPnPServiceProxyAction *
 gupnp_service_proxy_begin_action   (GUPnPServiceProxy              *proxy,
                                     const char                     *action,
@@ -186,16 +135,6 @@ gupnp_service_proxy_begin_action_list
                                     GUPnPServiceProxyActionCallback callback,
                                     gpointer                        user_data);
 
-#ifndef GUPNP_DISABLE_DEPRECATED
-GUPnPServiceProxyAction *
-gupnp_service_proxy_begin_action_hash
-                                   (GUPnPServiceProxy              *proxy,
-                                    const char                     *action,
-                                    GUPnPServiceProxyActionCallback callback,
-                                    gpointer                        user_data,
-                                    GHashTable                     *hash) G_GNUC_DEPRECATED;
-#endif
-
 gboolean
 gupnp_service_proxy_end_action     (GUPnPServiceProxy              *proxy,
                                     GUPnPServiceProxyAction        *action,
@@ -211,15 +150,6 @@ gupnp_service_proxy_end_action_valist
 
 gboolean
 gupnp_service_proxy_end_action_list
-                                  (GUPnPServiceProxy       *proxy,
-                                   GUPnPServiceProxyAction *action,
-                                   GError                  **error,
-                                   GList                   *out_names,
-                                   GList                   *out_types,
-                                   GList                  **out_values);
-
-gboolean
-gupnp_service_proxy_end_action_list_gi
                                   (GUPnPServiceProxy       *proxy,
                                    GUPnPServiceProxyAction *action,
                                    GList                   *out_names,
@@ -228,19 +158,12 @@ gupnp_service_proxy_end_action_list_gi
                                    GError                  **error);
 
 gboolean
-gupnp_service_proxy_end_action_hash_gi
+gupnp_service_proxy_end_action_hash
                                    (GUPnPServiceProxy              *proxy,
                                     GUPnPServiceProxyAction        *action,
                                     GHashTable                     *hash,
                                     GError                        **error);
 
-gboolean
-gupnp_service_proxy_end_action_hash
-                                   (GUPnPServiceProxy              *proxy,
-                                    GUPnPServiceProxyAction        *action,
-                                    GError                        **error,
-                                    GHashTable                     *hash);
-
 void
 gupnp_service_proxy_cancel_action  (GUPnPServiceProxy              *proxy,
                                     GUPnPServiceProxyAction        *action);
@@ -288,4 +211,4 @@ gupnp_service_proxy_get_subscribed (GUPnPServiceProxy              *proxy);
 
 G_END_DECLS
 
-#endif /* __GUPNP_SERVICE_PROXY_H__ */
+#endif /* GUPNP_SERVICE_PROXY_H */
